# -*- coding: utf-8 -*- 
# @Time    : 03/21/2024 
# @Author  : nzooherd 
# @File    : __init__.py 
# @Software: Visual Studio Code

from abc import ABCMeta
from functools import partial
from logging import getLogger

from sqlalchemy import false
from telethon._impl.client.events.event import Event
from telethon._impl.client.events.filters.combinators import Combinable

from aspect.api import ApiAspect
from component.pornherd_bot import PornherdBot
from utills.bytes_util import CallbackBytesRequest


logger = getLogger()

class BaseController(metaclass=ABCMeta):

    def __init__(self, mybot: PornherdBot) -> None:
        self.mybot = mybot
        self.register()

    def register(self):
        for api, event, handler in ApiAspect.HandlerApis: 
            if handler.__module__ == self.__module__:
                self.mybot.bot_client.add_event_handler(partial(handler, self), event, api.filter)
                logger.info(f"Register api: \"{api.filter}\" to handler: {handler.__module__}.{ handler.__name__}")


class CallbackPrefix(Combinable):
    """
    Filter by ``event.data`` using a full bytes match, used for events such as :class:`telethon.events.ButtonCallback`.

    It checks if ``event.data`` is equal to the data passed to the filter.

    :param data: Bytes to match data with.
    """

    __slots__ = ("prefix",)

    def __init__(self, prefix: str) -> None:
        self.prefix = prefix

    def __call__(self, event: Event) -> bool:
        data = getattr(event, "data", None)
        try:
            return CallbackBytesRequest.decode_api_path(data).startswith(self.prefix)
        except Exception as e:
            logger.error(f"CallbackPrefix error: {e}")
            return false

class RedeemMatch(Combinable):

    def __init__(self) -> None:
        super().__init__()

    def __call__(self, event: Event) -> bool:
        text = getattr(event, "text", None)
        if text is None:
            return False
        if len(text) != 20:
            return False
        
        for c in text:
            if c <= '9' and c >= '0':
                continue
            if c <= 'Z' and c >= 'A':
                continue
            if c <= 'z' and c >= 'a':
                continue
            return False
        
        return True
